package at.hl.okb.wsclient;

import static at.hl.okb.wsclient.Utilities.createNameFilter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import com.google.common.collect.Iterables;

import at.hl.wsclient.okb.Algorithm;
import at.hl.wsclient.okb.AlgorithmParameter;
import at.hl.wsclient.okb.ArrayOfAlgorithm;
import at.hl.wsclient.okb.ArrayOfAlgorithmParameter;
import at.hl.wsclient.okb.ArrayOfPlatform;
import at.hl.wsclient.okb.ArrayOfProblem;
import at.hl.wsclient.okb.IOKBService;
import at.hl.wsclient.okb.OKBService;
import at.hl.wsclient.okb.ObjectFactory;
import at.hl.wsclient.okb.Platform;
import at.hl.wsclient.okb.Problem;

public class OKBServiceClient {
    OKBService srv = null;
    IOKBService port = null;

    public OKBServiceClient() {
	this.srv = new OKBService();
	this.port = srv.getWSHttpBindingIOKBService();
    }

    public Long getPlatformId(String platformName) {
	Platform returnValue = Iterables.find(
		port.getPlatforms().getPlatform(),
		createNameFilter(platformName));
	if (returnValue == null)
	    createPlatform(platformName);

	returnValue = Iterables.find(port.getPlatforms().getPlatform(),
		createNameFilter(platformName));
	return returnValue.getId();

    }

    private void createPlatform(String name) {
	
	ObjectFactory factory = new ObjectFactory();
	Platform platform = factory.createPlatform();
	
	platform.setName(factory.createNamedOKBItemName(name));
	port.addPlatform(platform);
    }

    public Long getAlgorithmId(String algorithmName) {
	Long algorithmId = null;

	ArrayOfAlgorithm allAlgorithms = port.getAlgorithms();
	for (Algorithm algo : allAlgorithms.getAlgorithm()) {
	    if (algo.getName().getValue() == algorithmName) {
		algorithmId = algo.getId();
	    }
	}
	return algorithmId;
    }

    public Long getProblemId(String problemName) {
	Long problemId = null;

	ArrayOfProblem allProblems = port.getProblems();
	for (Problem problem : allProblems.getProblem()) {
	    if (problem.getName().getValue() == problemName) {
		problemId = problem.getId();
	    }
	}
	return problemId;
    }

    public void checkAlgoParamsExists(Long algorithmId,
	    Map<String, Object> ecjAlgoParams) {

	List<String> missingParams = new ArrayList<String>();

	ArrayOfAlgorithmParameter hlAlgoParams = port
		.getAlgorithmParameters(algorithmId);
	Map<String, Object> hlAlgoParamMap = new HashMap<String, Object>();

	for (AlgorithmParameter algoParam : hlAlgoParams
		.getAlgorithmParameter()) {
	    hlAlgoParamMap.put(algoParam.getName().getValue(), null); // value
								      // hangs
								      // on
								      // Experiments
	}

	String[] ecjparamNames = (String[]) ecjAlgoParams.keySet().toArray();

	for (int i = 0; i < ecjparamNames.length; i++) {
	    if (!hlAlgoParamMap.containsKey(ecjparamNames[i])) {
		missingParams.add(new String(ecjparamNames[i]));
	    }
	}

    }

}
